% clafic.m CLAFICによる個人識別
clear 
close all

r=1; % 部分空間の次元数 (各クラス共通, 1<=r<=9)

load('traiX.mat');    % traiのオリジナル画像とラベルの読込み
load('queryX.mat');   % queryのオリジナル画像とラベルの読込み

nclass=40; % クラス数
nquery=40; % テスト画像数

% 部分空間の直交系の算出
for jj = 1 : nclass
	% ラベルがjjの訓練画像をYに格納
	X=trai.X(:,find(trai.label==jj));	
	[d n] = size(X);           % Xのサイズ取得 n<<d
	N = X'*X;                  % n x nの小さい行列Nを計算
	rdim = rank(N);            % Nの階数を算出
	
	% 固有ベクトルを並べた行列Uの算出
	[eig_vec eig_val] = eig(N); % Nの固有値と固有ベクトルを求める
	[value index]=sort(diag(eig_val),'descend'); % 固有値を降順に並べ替え
	U = X * eig_vec(:,index(1:rdim)) * diag(sqrt(1./value(1:rdim)) );
	
	C(jj).U  = U; % クラスjjの固有ベクトル
	fprintf(2,'class %d --- ok\n',jj);
end

% CLAFICによるテストデータの識別
accuracy=0;          % 次元数毎の識別率
s=zeros(nclass,1);   % 各クラスの類似度
for ii = 1 : nquery
	
	% 各クラスの類似度を計算
	for jj = 1 : nclass
		s(jj)=norm((C(jj).U(:,1:r))'*query.X(:,ii));
	end
	
	% 類似度が最大となるクラスを求める
	[value index]=max(s);
	
	% 類似度が最大となるクラスとテストデータのラベルが等しい
	if index==query.label(ii)
		accuracy=accuracy+1;
	end
end
% 識別率を百分率で表現
accuracy=(accuracy./nquery).*100;
fprintf('dimensionality r=%d, accuracy : %2.1f \n',r,accuracy);
