clear 
close all
load('traiX.mat');
load('queryX.mat');
load('T.mat');

ntrai = size(trai.X,2); %標本数
d = size(trai.X,1); %次元数
nquery = length(query.label); %テスト標本数
nclass = max(trai.label);
imgsize = [112 92];

normalize=true; % 正規化
svmlight.v=0;
svmlight.z='c';
svmlight.c=100;  % ソフトマージンのパラメータ
svmlight.t=0; % 0: 線形，1:多項式(s a*b+c)^d，2:RBFexp(-gamma ||a-b||^2)，3:Sigmoid
svmlight.d=1; % 多項式のd
svmlight.g=1e-2; % RBFのgamma
svmlight.s=1; % 多項式/Sigmoidのs
svmlight.r=1; % 多項式/Sigmoidのc
svmlight.option=sprintf('-v %d -z %c -c %f -t %d -d %d -g %f -s %f -r %f ',...
			svmlight.v, svmlight.z, svmlight.c, svmlight.t, ...
			svmlight.d, svmlight.g, svmlight.s, svmlight.r);

%%% 正規化
if normalize 
  xnrm=sqrt(sum(trai.X.^2));
  trai.X = trai.X * diag( 1./xnrm);
  qnrm=sqrt(sum(query.X.^2));
  query.X = query.X * diag( 1./qnrm);
end

% SVMlight用のコマンド生成
if find(pwd==':')~=0 % windowsの場合
	svm_learn = 'svm_learn'; % svm_learn
	svm_classify = 'svm_classify'; % svm_classify
	trainfile='train.svmexample'; %学習標本一時ファイル
	testfile='test.svmexample'; %テスト標本一時ファイル
	modelfile='model.svmlight'; %モデル一時ファイル
	outputfile='output.svmlight'; %出力ファイル
else
	svm_learn = './svm_learn'; % svm_learn
	svm_classify = './svm_classify'; % svm_classify
	trainfile='./train.svmexample'; %学習標本一時ファイル
	testfile='./test.svmexample'; %テスト標本一時ファイル
	modelfile='./model.svmlight'; %モデル一時ファイル
	outputfile='./output.svmlight'; %出力ファイル
end

%% テスト標本をSVM-lightの形式で出力する
fid = fopen( testfile, 'w');
for ii=1:nquery
  fprintf(fid,'0 ');
  fprintf(fid,'%d:%e ',[ [1:d]; query.X(:,ii)' ] );
  fprintf(fid,'\n');
end
fclose(fid);

feature=zeros(nclass,nquery);
for c=1:nclass
  fprintf(2,'Class %d...',c);
  
  %% クラスがcならば1,それ以外ならば-1のベクトルを作成
  svmlabel = -1 * ones(1,ntrai);
  svmlabel( trai.label == c ) =1 ;
  
  %% SVM-lightの形式でデータを出力する
  fid = fopen( trainfile, 'w');
  for ii=1:ntrai
    fprintf(fid,'%d ', svmlabel(ii) );
    fprintf(fid,'%d:%e ',[ [1:d]; trai.X(:,ii)' ] );
    fprintf(fid,'\n');
  end
  fclose(fid);
  
  %% SVMの学習
  command = sprintf('%s %s %s %s\n', svm_learn, svmlight.option, trainfile, modelfile);
  [a b]=system(command);
  
  %% Support vectorの数
	fid=fopen(modelfile, 'r');
	str = fscanf( fid, '%c');
	fclose(fid);
	fprintf(2,'OK, # of SV = %d\n',sum(str==10)-11);
	clear str

  %% 識別
  command = sprintf('%s %s %s %s\n',svm_classify, testfile, modelfile, outputfile);
  [a b]=system(command);
  feature(c,:) = load(outputfile)';
  
end

[tmp eclass] = max(feature);
accuracy = sum(eclass(:) == query.label(:))/nquery;
fprintf('accuracy = %.2f %%\n',accuracy*100 );

%% 誤識別したものを表示
load('queryX.mat');
index = find( eclass(:) ~= query.label(:) );
for ii=1:min(length(index),9)
  subplot(3,3,ii);
  imagesc( reshape( T.m + T.U*query.X(:,index(ii)), imgsize(1), imgsize(2) ) );
  colormap('gray');
end
